// Atom Merge
// by Shadowdarke 12-13-01

var
	sd_EnableMerging = 0
	// prevents automatic merging until the world is created.

world/New()
	..()
	sd_Merge(world)	// automatically run once when the world is built
	sd_EnableMerging = 1

atom
	var/tmp/sd_MrgSet = 0	// which type of merge set the atom uses
						// 0 = no merging (default)
						// 1 = 16 states (adjacent 4 squares only)
						// 2 = 47 states (adjacent 4 and inner corners)
						// 3 = 256 states (all 8 directions)
	var/tmp/sd_EdgeMrg = 0	// if set, this atom will merge into the map edges

	New()
		..()
		if(sd_EnableMerging && sd_MrgSet)
			sd_Merge(sd_Adjacent())

	Del()
		if(sd_EnableMerging && sd_MrgSet)
			sd_DelayMerge(sd_Adjacent())
		..()

	proc
		sd_Adjacent()
		// returns a list of adjacent items relevant to the atom's MrgSet
			var/list
				adjacent = list()
				directions = list(NORTH,SOUTH,EAST,WEST)
			if(sd_MrgSet > 1)
				directions = list(NORTH,SOUTH,EAST,WEST,NORTHEAST,NORTHWEST,SOUTHEAST,SOUTHWEST)

			adjacent += src	// include src
			for(var/direction in directions)
				var/turf/T = get_step(src,direction)
				if(!T) continue
				if(isturf(src))
					adjacent += T
				else
					adjacent+= T.contents

			return adjacent

		sd_MergeMatch(direction)
			var/turf/T = get_step(src,direction)
			if(!T)
				return sd_EdgeMrg
			else
				if(T.type == src.type) return 1

	movable
		sd_MergeMatch(direction)
			var/turf/T = get_step(src,direction)
			if(!T)
				return sd_EdgeMrg
			else
				var/atom/movable/M
				for(M in T)
					if(M.type == src.type)
						return 1

proc/sd_DelayMerge(List)
	spawn(1) sd_Merge(List)

proc/sd_Merge(var/list/List)
	for(var/atom/A in List)
		if(!A.sd_MrgSet) continue	// don't need to merge if MrgSet = 0
		if(isarea(A)) continue	// sd_Merge() is not designed for areas.

		var/links = 0
		for(var/direction in list(NORTH,SOUTH,EAST,WEST))
			if(A.sd_MergeMatch(direction))
				links |= direction

		if(A.sd_MrgSet > 1) // effected by diagonals
			var/list/diagonals = list(NORTHEAST,SOUTHEAST,NORTHWEST,SOUTHWEST)
			for(var/X in 1 to diagonals.len)
				if(((links & diagonals[X])==diagonals[X]) || (A.sd_MrgSet==3))
					if(A.sd_MergeMatch(diagonals[X]))
						links |= (8 << X)

		A.icon_state = num2text(links)